/*
From hewat@ill.fr  Mon Feb 15 23:47:22 1993
Date: Tue, 16 Feb 93 08:40:31 -0100
I have modified the routine "cdmount" by Kent Landfield to allow all users 
to mount magneto-optic drives.  The rountines are now called "mountmo"
(you can mount a read-only disk with "mountmo -r"),  and "umountmo", 
where the buffer is dumped to the disk before dismounting it. (It must still
be manually ejected.)

Alan Hewat, ILL Grenoble, FRANCE (hewat@ill.fr) Fax (France=33).76.48.39.06
*/

/*
* Further modified 9/93 by Dave Olson to support ejecting media
* on unmount on SGI systems (-e option on umount)
* Requires -lds when compiling.
* Note that the binary must be called mountmo, with a link to umountmo
* install it mod 4711, owned by root.  See the MNTPOINT define for
* the mountpoint that is used (must be in fstab).
*/

/*
**  Subsystem:   User Level mount for CD-ROM modified for magneto-optic
**  File Name:   mountmo.c
**                                                        
** This software is Copyright (c) 1991 by Kent Landfield.
** based on cdmount, modified by Alan Hewat for magneto-optic drives
**
** Permission is hereby granted to copy, distribute or otherwise 
** use any part of this package as long as you do not try to make 
** money from it or pretend that you wrote it.  This copyright 
** notice must be maintained in any copy made.
**
** Use of this software constitutes acceptance for use in an AS IS 
** condition. There are NO warranties with regard to this software.  
** In no event shall the author be liable for any damages whatsoever 
** arising out of or in connection with the use or performance of this 
** software.  Any use of this software is at the user's own risk.
**
**  If you make modifications to this software that you feel 
**  increases it usefulness for the rest of the community, please 
**  email the changes, enhancements, bug fixes as well as any and 
**  all ideas to me. This software is going to be maintained and 
**  enhanced as deemed necessary by the community.
**              
**              Kent Landfield
**              sparky!kent
**              kent@sparky.imd.sterling.com
**
**              Alan Hewat
**              hewat@ill.fr
*/

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include <mntent.h>
#include <string.h>
#include <malloc.h>
#include <dslib.h>

#define MNTPOINT "/modrive"
int debug;

char *
finddev(char *mntpnt)
{
	FILE *mtab = setmntent("/etc/mtab", "r");
	struct mntent *mnt;
	char *mntdev;

	while((mnt=getmntent(mtab)) && strcmp(mnt->mnt_dir, mntpnt))
		;
	if(!mnt) {
		endmntent(mtab);
		return NULL;	/* couldn't find mount point in mtab */
	}

	/* hack, hack.  Only deal with scsi disks; either via dks
	 * or ds.  dks naming standard is:
		dks#d#xxxx
	 * where first # is 1 or 2 digits for controller, and
	 * second is 1 or 2 digits of target ID.  Rest is partition
	 * identifier. */
	if(strncmp("/dev/scsi/", mnt->mnt_fsname, 10) == 0)
		mntdev = strdup(mnt->mnt_fsname);
	else if(strncmp("/dev/dsk/dks", mnt->mnt_fsname, 12) == 0 &&
		(mntdev = malloc(32))) {
		int ctlr, target;
		char *nxt;
		ctlr = strtoul(mnt->mnt_fsname+12, &nxt, 10);
		if(nxt)
			target = strtoul(nxt+1, &nxt, 10);
		if(nxt)
			sprintf(mntdev, "/dev/scsi/sc%dd%dl0", ctlr, target);
		else {
			free(mntdev);
			mntdev = NULL;
		}
	}
	else
		mntdev = NULL;

	endmntent(mtab);
	return mntdev;
}

void
ejectmedia( struct dsreq *dsp )
{
	fillg0cmd( dsp, CMDBUF( dsp ),  G0_LOAD, 1, 0, 0, 2, 0 );
	filldsreq( dsp, NULL, 0, DSRQ_READ|DSRQ_SENSE );
	doscsireq( getfd( dsp ), dsp );
}

/* could use this on mount to disable eject button someday */
void
prevent_removal( struct dsreq *dsp )
{
	fillg0cmd( dsp, CMDBUF( dsp ), G0_PREV, 0, 0, 0, 1, 0 );
	filldsreq( dsp, NULL, 0, DSRQ_READ|DSRQ_SENSE );
	doscsireq( getfd( dsp ), dsp );
}

void
allow_removal( struct dsreq *dsp )
{
	fillg0cmd( dsp, CMDBUF( dsp ), G0_PREV, 0, 0, 0, 0, 0 );
	filldsreq( dsp, NULL, 0, DSRQ_READ|DSRQ_SENSE );
	doscsireq( getfd( dsp ), dsp );
}


ejectit(char *scdev)
{
	dsreq_t         *dsp;
	int             ret, retries;

	dsdebug = debug;

	dsp = dsopen(scdev, O_RDONLY);
	if (!dsp) {
		fprintf(stderr, "can't open %s to eject media\n", scdev);
		return 2;
	}

	/* some drives don't allow software eject if prevent media removal
	 * has been done, so be paranoid */
	allow_removal(dsp);

	ejectmedia( dsp );
	ret = (STATUS( dsp ) == STA_GOOD ? 0 : 3);
	if (ret != 0)
		fprintf( stderr, "Can't eject media from %s\n", scdev );
	else if(debug)
		fprintf( stderr, "ejected media from %s\n", scdev );
	dsclose( dsp );
	return (ret);
}


void
usage(char *progname)
{
    (void) fprintf(stderr, "\nusage: %s [ -dhve ]\
\n\noptions:\n\
      -d   show the mount command without executing it\n\
      -r   mount the media readonly (mount only)\n\
      -v   show the mount command and execute it\n\
      -e   eject the media after unmounting, if successful (unmount only)\n\
\n", progname);
}

int main(argc, argv)
int argc;
char **argv;
{
    char *cp;
    char cmd[256];
    int rc;
    int readonly = 0;
	int doeject = 0;
    struct stat stb;
    extern char *optarg;
    extern int optind;
    extern int opterr;

	opterr = 0;

    if ((cp = strrchr(argv[0],'/')) != NULL)
        ++cp;
    else 
        cp = argv[0];

    /*
    ** Setup IFS for system() protection...
    */
    if (putenv("IFS= \t\n") != 0) {
         (void) fprintf(stderr,"%s: IFS putenv failed...\n", cp);
         return(1);
    }

    /*
    ** Setup PATH for execlp() protection...
    */
    if (putenv("PATH=/etc:/usr/etc:/bin:/usr/bin") != 0) {
         (void) fprintf(stderr,"%s: PATH putenv failed...\n", cp);
         return(1);
    }

    /*
    ** If the user is requesting to mount an MO..
    */
    if (strcmp(cp, "mountmo") == 0) {
        /*
        ** Assure that the mount point is there and in a 
        ** directory and not a symbolic link ..
        */
        if (lstat(MNTPOINT, &stb) != 0) {
            (void) fprintf(stderr, "%s: mount point missing\n", MNTPOINT);
            return(1);
        }
        if ((stb.st_mode & S_IFMT) != S_IFDIR) {
            (void) fprintf(stderr, "%s: invalid mount point\n", MNTPOINT);
            return(1);
        }

        if (argc > 1) {
           while ((rc = getopt(argc, argv, "drv")) != EOF) {
              switch (rc) {
                  case 'd':     /* debugging - does not run command. */
                      debug = 1;
                      break;
                  case 'v':     /* verbose - runs command. */
                      debug = 2;
                      break;
                  case 'r':     /* read-only */
                      readonly++;
                      break;
                  default:
                      usage(cp);
                      return(1);
              }
           }   
        }   

        /* build the command line.. */
        
        (void) sprintf(cmd, "/etc/mount %s -o nosuid %s",
                       readonly ? "-r" : "", MNTPOINT);
        if (debug) 
            (void) fprintf(stderr, "%s\n", cmd);
        if (debug != 1)
            rc = system(cmd);
    }

    else if (strcmp(cp, "umountmo") == 0) {
		/* unmount it */
		if (argc > 1) {
		   while ((rc = getopt(argc, argv, "vde")) != EOF) {
			  switch (rc) {
				  case 'd':     /* debugging - does not run command. */
					  debug = 1;
					  break;
				  case 'v':     /* verbose - runs command. */
					  debug = 2;
					  break;
				  case 'e':     /*  eject after unmount */
					  doeject = 1;
					  break;
				  default:
					  usage(cp);
					  return(1);
				}
			}
		}
		(void) sprintf(cmd, "/etc/umount %s", MNTPOINT);
		if (debug) 
			(void) fprintf(stderr, "%s\n", cmd);
		if (debug != 1) {
			char *ejectdev;
			if(doeject)
				ejectdev = finddev(MNTPOINT);
			rc = system(cmd);
			if(rc)
				fprintf(stderr, "unmount failed, if busy, try /etc/fuser\n");
			else if(doeject && ejectdev)
				(void)ejectit(ejectdev);
		}
	}

    /*
    ** Improperly named/linked executables, I'm confused...
    */
    else {
        (void) fprintf(stderr, "%s: I don't know who I am (want mountmo or umountm)\n", cp);
        rc = 1;
    }
    return(rc >> 8);
}
